/**
 * @file debug.h
 * @brief Macros das funes de depurao
 *
 * Macros que sero usadas nos programas desenvolvidos ao longo dos
 * exemplos. Estas macros podem receber um nmero varivel de parmetros
 * atravs de uma string de formatao como no "printf". Seguem-se alguns 
 * exemplos: 
 * @code
 * DEBUG("i = %d e f=.2f%", i, f);
 * ERROR("%s", msg);
 * @endcode
 * @author Miguel Frade, Patricio Domingues, Vitor Carreira
 * @date Agosto de 2003
 * @version 2
 */
#ifndef DEBUG_H
#define DEBUG_H

void debug(const char *file, const int line, char *fmt, ...);
void warning(const char *file, const int line, char *fmt, ...);
void error(const char *file, const int line, int exitCode, char *fmt, ...);
void h_warning(const char *file, const int line, char *fmt, ...);
void h_error(const char *file, const int line, int exitCode, char *fmt,
	     ...);


/**
 * Macro para imprimir no stderr informaes teis 
 * para depurao. O nmero de parmetros de entrada
 *  varivel.
 * 
 * @return A funo no retorna nada
 * @see debug()
 */
#define DEBUG(...) debug(__FILE__, __LINE__, __VA_ARGS__)


/**
 * Macro para imprimir no stderr informao relacionada
 * com insucesso de chamadas de funes, mas no termina a
 * execuo do programa. O nmero de parmetros de entrada
 *  varivel.
 *
 * @return A funo no retorna nada
 * @see warning()
 */
#define WARNING(...) warning(__FILE__, __LINE__, __VA_ARGS__)


/**
 * Macro para imprimir no stderr informao relacionada
 * com insucesso de chamadas de funes e termina a execuo
 * do programa. O nmero de parmetros de entrada  varivel.
 *
 * @return A funo no retorna nada
 * @see error()
 */
#define ERROR(exitCode, ...) \
	error(__FILE__, __LINE__, (exitCode), __VA_ARGS__)


/**
 * Macro para imprimir no stderr informao relacionada
 * com insucesso de chamadas de funes de resoluo de nomes,
 * mas no termina a execuo do programa. O nmero de 
 * parmetros de entrada  varivel.
 *
 * @return A funo no retorna nada
 * @see h_warning()
 */
#define H_WARNING(...) h_warning(__FILE__, __LINE__, __VA_ARGS__)


/**
 * Macro para imprimir no stderr informao relacionada
 * com insucesso de chamadas de funes de resoluo de nomes 
 * e termina a execuo do programa. O nmero de parmetros
 * de entrada  varivel.
 *
 * @return A funo no retorna nada
 * @see h_error()
 */
#define H_ERROR(exitCode, ...) \
	h_error(__FILE__, __LINE__, (exitCode), __VA_ARGS__)


#endif				/* DEBUG_H */
